local Grid2StatusAuraGroup = Grid2:NewModule("StatusAuraGroup")


--{{{  Status Creation

-- UpdateState for all Auras in the Group
local function status_UpdateState(self, unit, auraName, iconTexture, count, duration, expiration)
	if (self.auraNames[auraName]) then
		self.states[unit] = true
		self.textures[unit] = self.iconTexture
		self.counts[unit] = count
		self.durations[unit] = duration
		self.expirations[unit] = expiration
		self.new_state = self:IsActive(unit)
		self.new_count = count
		self.new_expiration = expiration
	end
end

-- UpdateState for only my Auras in the Group
local function status_UpdateStateMine(self, unit, auraName, iconTexture, count, duration, expiration, isMine)
	if (self.auraNames[auraName] and isMine) then
		self.states[unit] = true
		self.textures[unit] = self.iconTexture
		self.counts[unit] = count
		self.durations[unit] = duration
		self.expirations[unit] = expiration
		self.new_state = self:IsActive(unit)
		self.new_count = count
		self.new_expiration = expiration
	end
end

-- Buff type group
function Grid2:SetupStatusAuraGroupBuff(infoAuraGroup)
	local info = infoAuraGroup[1]
	local status = self:CreateBuffStatus(unpack(info))

	local mine = info[2]
	status.UpdateState = mine and status_UpdateStateMine or status_UpdateState

	return status
end

-- Debuff type group
function Grid2:SetupStatusAuraGroupDebuff(infoAuraGroup)
	local info = infoAuraGroup[1]
	local status = self:CreateDebuffStatus(unpack(info))

	status.UpdateState = status_UpdateState

	return status
end

-- AuraGroup is implemented as a regular Aura status with modified detection code for UpdateState
function Grid2:SetupStatusAuraGroup(status, statusKey, infoAuraGroup)
	local info = infoAuraGroup[1]
	local displaySpellId = info[1]
	status.name = statusKey -- force name

	status.defaultDB.profile.displaySpellId = displaySpellId
	_, _, status.iconTexture = GetSpellInfo(displaySpellId)

	local auraNames = {}
	status.auraNames = auraNames
	for index = 2, # infoAuraGroup do
		local spellName = infoAuraGroup[index]
		if (type(spellName) == "number") then
			spellName = GetSpellInfo(spellName)
		end
		assert(type(spellName) == "string")
		auraNames[spellName] = true
	end

	self:RegisterStatus(status, { "color", "icon", "percent" })
	self:MakeBuffColorHandler(status)
	status:UpdateProfileData()
	return status
end


-- Hook SetupAuraStatus to all StatusAuraGroup in the setup
local prev_SetupAuraStatus = Grid2.SetupAuraStatus
function Grid2:SetupAuraStatus(setup, ...)
	prev_SetupAuraStatus(self, setup, ...)

	local status
	for statusKey, infoAuraGroup in pairs(setup.auraGroups) do
		status = self:SetupStatusAuraGroupBuff(infoAuraGroup)
		self:SetupStatusAuraGroup(status, statusKey, infoAuraGroup)
	end
	for statusKey, infoAuraGroup in pairs(setup.auraGroupDebuffs) do
		status = self:SetupStatusAuraGroupDebuff(infoAuraGroup)
		self:SetupStatusAuraGroup(status, statusKey, infoAuraGroup)
	end

--test()
end

--}}}


-- Hook the loading of options so our associated lod options get loaded at the right time.
local prev_LoadOptions = Grid2.LoadOptions
function Grid2:LoadOptions(...)
	prev_LoadOptions(self, ...)
	if (Grid2StatusAuraGroupOptions) then
		return
	end
	if (not IsAddOnLoaded("Grid2StatusAuraGroupOptions")) then
		LoadAddOn("Grid2StatusAuraGroupOptions")
	end
end


-- Hook GetCurrentSetup to blend in default options.
local prev_GetCurrentSetup = Grid2.GetCurrentSetup
function Grid2:GetCurrentSetup(...)
	local setup, class = prev_GetCurrentSetup(self, ...)

	if (Grid2StatusAuraGroupOptions) then
		Grid2StatusAuraGroupOptions:MakeDefaultSetup(setup, class)
	end

	return setup, class
end
